Stan Math Library: stan/math/prim/scal/prob/logistic_ccdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

logistic_ccdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_LOGISTIC_CCDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_LOGISTIC_CCDF_LOG_HPP
3
4 #include <boost/random/exponential_distribution.hpp>
5 #include <boost/random/variate_generator.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/value_of.hpp>
13 #include <stan/math/prim/scal/fun/log1p.hpp>
14 #include <stan/math/prim/scal/meta/length.hpp>
15 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
16 #include <stan/math/prim/scal/meta/VectorView.hpp>
17 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
18 #include <stan/math/prim/scal/meta/contains_nonconstant_struct.hpp>
19 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
20 #include <stan/math/prim/scal/meta/return_type.hpp>
21 #include <stan/math/prim/scal/prob/logistic_log.hpp>
22 #include <stan/math/prim/scal/meta/include_summand.hpp>
23 #include <cmath>
24 #include <limits>
25
26 namespace stan {
27 namespace math {
28
29 template <typename T_y, typename T_loc, typename T_scale>
30 typename return_type<T_y, T_loc, T_scale>::type
31 logistic_ccdf_log(const T_y& y, const T_loc& mu, const T_scale& sigma) {
32 typedef typename stan::partials_return_type<T_y, T_loc, T_scale>::type
33 T_partials_return;
34
35 // Size checks
36 if ( !( stan::length(y) && stan::length(mu) && stan::length(sigma) ) )
37 return 0.0;
38
39 // Error checks
40 static const char* function("stan::math::logistic_cdf_log");
41
42 using stan::math::check_not_nan;
43 using stan::math::check_positive_finite;
44 using stan::math::check_finite;
45 using stan::math::check_consistent_sizes;
46 using stan::math::value_of;
47 using boost::math::tools::promote_args;
48 using std::log;
49 using std::exp;
50
51 T_partials_return P(0.0);
52
53 check_not_nan(function, "Random variable", y);
54 check_finite(function, "Location parameter", mu);
55 check_positive_finite(function, "Scale parameter", sigma);
56 check_consistent_sizes(function,
57 "Random variable", y,
58 "Location parameter", mu,
59 "Scale parameter", sigma);
60
61 // Wrap arguments in vectors
62 VectorView<const T_y> y_vec(y);
63 VectorView<const T_loc> mu_vec(mu);
64 VectorView<const T_scale> sigma_vec(sigma);
65 size_t N = max_size(y, mu, sigma);
66
67 OperandsAndPartials<T_y, T_loc, T_scale>
68 operands_and_partials(y, mu, sigma);
69
70 // Explicit return for extreme values
71 // The gradients are technically ill-defined, but treated as zero
72
73 for (size_t i = 0; i < stan::length(y); i++) {
74 if (value_of(y_vec[i]) == -std::numeric_limits<double>::infinity())
75 return operands_and_partials.value(0.0);
76 }
77
78 // Compute vectorized cdf_log and its gradients
79 for (size_t n = 0; n < N; n++) {
80 // Explicit results for extreme values
81 // The gradients are technically ill-defined, but treated as zero
82 if (value_of(y_vec[n]) == std::numeric_limits<double>::infinity()) {
83 return operands_and_partials.value(stan::math::negative_infinity());
84 }
85
86 // Pull out values
87 const T_partials_return y_dbl = value_of(y_vec[n]);
88 const T_partials_return mu_dbl = value_of(mu_vec[n]);
89 const T_partials_return sigma_dbl = value_of(sigma_vec[n]);
90 const T_partials_return sigma_inv_vec = 1.0 / value_of(sigma_vec[n]);
91
92 // Compute
93 const T_partials_return Pn = 1.0 - 1.0 / (1.0 + exp(-(y_dbl - mu_dbl)
94 * sigma_inv_vec));
95 P += log(Pn);
96
97 if (!is_constant_struct<T_y>::value)
98 operands_and_partials.d_x1[n]
99 -= exp(logistic_log(y_dbl, mu_dbl, sigma_dbl)) / Pn;
100 if (!is_constant_struct<T_loc>::value)
101 operands_and_partials.d_x2[n]
102 -= - exp(logistic_log(y_dbl, mu_dbl, sigma_dbl)) / Pn;
103 if (!is_constant_struct<T_scale>::value)
104 operands_and_partials.d_x3[n] -= - (y_dbl - mu_dbl) * sigma_inv_vec
105 * exp(logistic_log(y_dbl, mu_dbl, sigma_dbl)) / Pn;
106 }
107
108 return operands_and_partials.value(P);
109 }
110 }
111 }
112 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::logistic_ccdf_log
return_type< T_y, T_loc, T_scale >::type logistic_ccdf_log(const T_y &y, const T_loc &mu, const T_scale &sigma)
Definition: logistic_ccdf_log.hpp:31
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
contains_nonconstant_struct.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
return_type.hpp
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
logistic_log.hpp
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
stan::math::logistic_log
return_type< T_y, T_loc, T_scale >::type logistic_log(const T_y &y, const T_loc &mu, const T_scale &sigma)
Definition: logistic_log.hpp:32
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
log1p.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::negative_infinity
double negative_infinity()
Return negative infinity.
Definition: constants.hpp:132

[ Stan Home Page ] © 2011–2016, Stan Development Team.